#!/usr/bin/env python3

import json
import os
from scanner import scanner, dupecheck, imageboard_info, downloader

print("Testing scanner.py")

print("--------------------------------------------------------")
print("Testing: scanner.get_catalog_json                      -")
print("--------------------------------------------------------")
catalog_json = scanner.get_catalog_json("a", "4chan")
print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("Testing: scanner.scan_thread                           -")
print("--------------------------------------------------------")

list_of_threads = scanner.scan_thread("anime", catalog_json)
for i in list_of_threads:
     print(i)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("!!! scanner.download_thread not tested yet !!!         -")
print("--------------------------------------------------------")

print("--------------------------------------------------------")
print("Testing: scanner.folder_size_mb                        -")
print("--------------------------------------------------------")

#Creating a 15mb file in a subfolder of a folder
os.system("mkdir folder_size_test")
os.system("mkdir folder_size_test/subfolder")
os.system("dd if=/dev/zero of=folder_size_test/subfolder/15mbfile bs=15000000 count=1")

# Getting folder size
size = scanner.folder_size_mb("folder_size_test")
if int(size) != 15:
    print(size)
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("!!! scanner.check_quota not tested yet !!!             -")
print("--------------------------------------------------------")

print("--------------------------------------------------------")
print("Testing:                                               -")
print("scanner.get_check_duplicate                            -")
print("scanner.get_condition                                  -")
print("scanner.get_imageboard                                 -")
print("scanner.get_keyword                                    -")
print("--------------------------------------------------------")

json_file = json.load(open("test/test_config.json"))

# Get a list of every search entry in the json file
search_list = []
for search in json_file["searches"]:
    search_list.append(search)

# Test on the first search with all optionals parameters
duplicate1  = scanner.get_check_duplicate(search_list[0])
condition1  = scanner.get_condition(search_list[0])
imageboard1 = scanner.get_imageboard(search_list[0])
keyword1    = scanner.get_keyword(search_list[0])

if not duplicate1:
    print("duplicate1 should be True but is False")
    exit(1)

if condition1["filename"] != ['IMG_']:
    print("filename error in condition1")
    exit(1)

if condition1["width"] != '>100':
    print("width error in condition1")
    exit(1)

if condition1["height"] != '>200':
    print("height error in condition1")
    exit(1)

if condition1["ext"] != ['.jpg', '.png']:
    print("ext error in condition1")
    exit(1)

if imageboard1 != '4chan':
    print("imageboard1 should be 4chan")
    exit(1)

if keyword1 != ['keyword1', 'keyword2', 'keyword3']:
    print("keyword1 should be equal to ['keyword1', 'keyword2', 'keyword3']")
    exit(1)

duplicate2  = scanner.get_check_duplicate(search_list[1])
condition2  = scanner.get_condition(search_list[1])
imageboard2 = scanner.get_imageboard(search_list[1])
keyword2    = scanner.get_keyword(search_list[1])

if not duplicate2:
    print("duplicate2 should be True but is False")
    exit(1)

if condition2["filename"]:
    print("filename should be false")
    exit(1)

if condition2["width"]:
    print("width should be false")
    exit(1)

if condition2["height"]:
    print("height should be false")
    exit(1)

if condition2["ext"]:
    print("ext should be false")
    exit(1)

if imageboard2 != '4chan':
    print("imageboard2 should be 4chan")
    exit(1)

if keyword2 != ['keyword']:
    print("keyword2 should be equal to ['keyword']")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("!!! scanner.scan not tested yet !!!                    -")
print("--------------------------------------------------------")

print('\x1b[6;30;42m' + 'All test OK for scanner.py' + '\x1b[0m')

print("Testing dupecheck.py")

print("--------------------------------------------------------")
print("Testing: dupecheck.hash_image                          -")
print("--------------------------------------------------------")

hash = dupecheck.hash_image("test/test_img.png")
if hash != "b3ce9cb3aefc5e240b4295b406ce8b9a":
    print("hash should be b3ce9cb3aefc5e240b4295b406ce8b9a")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("!!! dupecheck.add_to_db not tested yet !!!             -")
print("--------------------------------------------------------")

print("--------------------------------------------------------")
print("!!! dupecheck.is_duplicate not tested yet !!!          -")
print("--------------------------------------------------------")

print('\x1b[6;30;42m' + 'All test OK for dupecheck.py' + '\x1b[0m')

print("Testing imageboard_info.py")

print("--------------------------------------------------------")
print("Testing: imageboard_info.get_imageboard_info           -")
print("--------------------------------------------------------")

info_4chan = imageboard_info.imageboard_info("4chan")
if info_4chan.base_url != "http://a.4cdn.org/":
    print("chan_base_url wrong for 4chan")
    exit(1)

if info_4chan.thread_subfolder != "/thread/":
    print("chan_thread_subfolder wrong for 4chan")
    exit(1)

if info_4chan.image_subfolder != "/":
    print("chan_image_subfolder wrong for 4chan")
    exit(1)

if info_4chan.image_base_url != "http://i.4cdn.org/":
    print("chan_image_base_url wrong for 4chan")
    exit(1)

info_lainchan = imageboard_info.imageboard_info("lainchan")
if info_lainchan.base_url != "https://lainchan.org/":
    print("chan_base_url wrong for lainchan")
    exit(1)

if info_lainchan.thread_subfolder != "/res/":
    print("chan_thread_subfolder wrong for lainchan")
    exit(1)

if info_lainchan.image_subfolder != "/src/":
    print("chan_image_subfolder wrong for lainchan")
    exit(1)

if info_lainchan.image_base_url != "https://lainchan.org/":
    print("chan_image_base_url wrong for lainchan")
    exit(1)


info_uboachan = imageboard_info.imageboard_info("uboachan")
if info_uboachan.base_url != "https://uboachan.net/":
    print("chan_base_url wrong for uboachan")
    exit(1)

if info_uboachan.thread_subfolder != "/res/":
    print("chan_thread_subfolder wrong for uboachan")
    exit(1)

if info_uboachan.image_subfolder != "/src/":
    print("chan_image_subfolder wrong for uboachan")
    exit(1)

if info_uboachan.image_base_url != "https://uboachan.net/":
    print("chan_image_base_url wrong for uboachan")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print('\x1b[6;30;42m' + 'All test OK for imageboard_info.py' + '\x1b[0m')

print("Testing download.py")

# Creating download object
condition = {"ext": False, "filename": False, "width": False, "height": False, }
download = downloader.downloader(list_of_threads[0], 'a',"4chan", ".", "testci", True, condition, True)

print("--------------------------------------------------------")
print("!!! download.load not tested yet !!!                   -")
print("--------------------------------------------------------")

print("--------------------------------------------------------")
print("Testing: download.create_dir                           -")
print("--------------------------------------------------------")

# Creating directory
download.create_dir("test_create_dir")
if not os.path.exists("test_create_dir"):
    print("'test_create_dir' was not created")
    exit(1)

# Testing again because the function should not crash if folder already exist
download.create_dir("test_create_dir")

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("Testing: download.add_to_downloaded_log                -")
print("Testing: download.was_downloaded                       -")
print("--------------------------------------------------------")

os.system("echo "" > test_download_log.txt")
download.add_to_downloaded_log("my_filename")
if 'my_filename' not in open(download.downloaded_log).read():
    print("'my_filename' is not in {0}".format(download.downloaded_log))
    exit(1)

downloaded = download.was_downloaded("my_filename")
if not downloaded:
    print("'returned' should be True")
    exit(1)

downloaded = download.was_downloaded("other_filename")
if downloaded:
    print("'returned' should be False")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("Testing: download.extension_condition                  -")
print("--------------------------------------------------------")

if not download.extension_condition([".jpg"], ".jpg"):
    print("same extension should return True")
    exit(1)

if download.extension_condition([".jpg"], ".png"):
    print("different extension should return False")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("Testing: download.filename_condition                   -")
print("--------------------------------------------------------")

if not download.filename_condition(["IMG_"], "IMG_2345.jpg"):
    print("IMG_ is in IMG_2345, should return True")
    exit(1)

if download.filename_condition(["PIC"], "IMG_2345.jpg"):
    print("PIC is not in IMG_2345, should return False")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("Testing: download.width_condition                      -")
print("--------------------------------------------------------")

if not download.width_condition("=100", 100):
    print("100 is equal to 100, should be True")
    exit(1)

if download.width_condition("=100", 101):
    print("100 is not equal to 101, should be False")
    exit(1)

if not download.width_condition(">100", 101):
    print("101 is greater than 100, should be True")
    exit(1)

if download.width_condition(">100", 99):
    print("99 is not greater than 100, should be False")
    exit(1)

if not download.width_condition("<100", 99):
    print("99 is lower than 100, should be True")
    exit(1)

if download.width_condition("<100", 101):
    print("101 is not lower than 100, should be False")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')


print("--------------------------------------------------------")
print("Testing: download.all_condition_check                  -")
print("--------------------------------------------------------")

all_true = [True, True, True, True]
one_false = [True, False, True, True]
all_false = [False, False, False, False]

if not download.all_condition_check(all_true):
    print("all conditions are True, should return True")
    exit(1)

if download.all_condition_check(one_false):
    print("one condition is False, should return False")
    exit(1)

if download.all_condition_check(all_false):
    print("all conditions are False, should return False")
    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("!!! download.meet_dl_condition not tested yet !!!      -")
print("--------------------------------------------------------")

print("--------------------------------------------------------")
print("!!! download.remove_if_duplicate not tested yet !!!    -")
print("--------------------------------------------------------")

print("--------------------------------------------------------")
print("!!! download.download_image not tested yet !!!         -")
print("--------------------------------------------------------")

#img_url= "https://github.com/Lacsap-/4scanner/raw/master/logo/"
#post_dic = {'tim': '4scanner128', 'ext': '.png'}

#file_path = download.download_image(img_url, post_dic, ".")
#if not os.path.isfile(file_path):
#    print("4scanner128.png should have been downloaded.")
#    exit(1)

print('\x1b[6;30;42m' + 'OK' + '\x1b[0m')

print("--------------------------------------------------------")
print("!!! download.download_thread not tested yet !!!        -")
print("--------------------------------------------------------")

print('\x1b[6;30;42m' + 'All test OK for download.py' + '\x1b[0m')
print('\x1b[6;30;42m' + 'SUCCESS' + '\x1b[0m')



#!/usr/bin/env python3

class imageboard_info:

    def __init__(self, imageboard):
        if imageboard == "4chan":
            self.base_url = "http://a.4cdn.org/"
            self.image_base_url = "http://i.4cdn.org/"
            self.thread_subfolder = "/thread/"
            self.image_subfolder = "/"
        elif imageboard == "lainchan":
            self.base_url = "https://lainchan.org/"
            self.image_base_url = "https://lainchan.org/"
            self.thread_subfolder = "/res/"
            self.image_subfolder = "/src/"
        elif imageboard == "uboachan":
            self.base_url = "https://uboachan.net/"
            self.image_base_url = "https://uboachan.net/"
            self.thread_subfolder = "/res/"
            self.image_subfolder = "/src/"
        else:
            raise ValueError("Imageboard {0} is not supported.".format(imageboard))



#!/usr/bin/env python3


import json
import logging
import os
from scanner import imageboard_info, dupecheck
from scanner.config import DB_FILE, currently_downloading
import sqlite3
import sys
import re
import time
import urllib
import http.client
import requests
import threading
import shutil

class downloader:

    def __init__(self, thread_nb, board, imageboard, output_folder, folder, is_quiet, condition, check_duplicate):
        # Getting info about the imageboard URL
        ib_info = imageboard_info.imageboard_info(imageboard)

        base_url = ib_info.base_url
        image_url = ib_info.image_base_url
        thread_subfolder = ib_info.thread_subfolder
        image_subfolder = ib_info.image_subfolder

        # These URL are the url of the thread
        # and the base url where images are stored on the imageboard
        self.thread_url = "{0}{1}{2}{3}.json".format(base_url, board, thread_subfolder, thread_nb)
        self.image_url = "{0}{1}{2}".format(image_url, board, image_subfolder)

        self.tmp_dir = "/tmp/{0}/".format(os.getpid())
        self.curr_time = time.strftime('%d%m%Y-%H%M%S')
        self.pid = os.getpid()
        self.thread = threading.current_thread().name

        self.downloaded_log = "{0}/{1}4scanner_dld-{2}-{3}".format(self.tmp_dir, self.curr_time, self.pid, self.thread)

        self.out_dir = os.path.join(output_folder, 'downloads', imageboard, board, folder, str(thread_nb))

        self.thread_nb = thread_nb
        self.imageboard = imageboard
        self.board = board
        self.condition = condition
        self.check_duplicate = check_duplicate
        self.is_quiet = is_quiet

        # Creating the tmp and output directory
        self.create_dir(self.tmp_dir)
        self.create_dir(self.out_dir)


    # Main download function
    def download(self):
        while True:
            # Getting the thread's json
            try:
                thread_json = json.loads(self.get_thread_json())
            except ValueError:
                print("Problem connecting to {0}. stopping download for thread {1}".format(self.imageboard, self.thread_nb))
                self.remove_thread_from_downloading()
                self.remove_tmp_files()
                exit(1)

            # Image download loop
            for post in thread_json["posts"]:
                if 'filename' in post:
                    if not self.was_downloaded(post["tim"]):
                        if self.meet_dl_condition(post):
                            tmp_pic = self.download_image(post)
                            final_pic = os.path.join(self.out_dir, tmp_pic.split('/')[-1])
                            self.add_to_downloaded_log(post["tim"])

                            if self.check_duplicate:
                                # If picture is not a duplicate copy it to out_dir
                                if not self.remove_if_duplicate(tmp_pic):
                                    shutil.move(tmp_pic, final_pic)
                            else:
                                shutil.move(tmp_pic, final_pic)

                            time.sleep(2)

                # Some imageboards allow more than 1 picture per post
                if 'extra_files' in post:
                    for picture in post["extra_files"]:
                        if not self.was_downloaded(picture["tim"]):
                            if self.meet_dl_condition(picture):
                                tmp_pic = self.download_image(picture)
                                final_pic = os.path.join(out_dir, tmp_pic.split('/')[-1])
                                self.add_to_downloaded_log(picture["tim"])

                                if self.check_duplicate:
                                    # If picture is not a duplicate copy it to out_dir
                                    if not self.remove_if_duplicate(tmp_pic):
                                        shutil.move(tmp_pic, final_pic)
                                else:
                                    shutil.move(tmp_pic, final_pic)


                                time.sleep(2)
            if not self.is_quiet:
                print('.')
            time.sleep(20)


    def remove_thread_from_downloading(self):
        # In a try except because 4downloader do not store threads in this list
        try:
            scanner.currently_downloading.remove(self.thread_nb)
        except NameError as e:
            pass


    def add_thread_to_downloaded(self):
        conn = sqlite3.connect(DB_FILE)
        c = conn.cursor()

        c.execute("INSERT INTO Downloaded_Thread (Thread_Number, Imageboard, Board) VALUES (?, ?, ?)",
                 (self.thread_nb, self.imageboard, self.board))

        conn.commit()
        conn.close()


    def get_thread_json(self):
        response = requests.get(self.thread_url)
        if response.status_code == 404:
            if not self.is_quiet:
                print('thread 404\'d')
            self.remove_thread_from_downloading()
            self.add_thread_to_downloaded()
            exit(0)
        return response.text


    def create_dir(self, directory):
        if not os.path.exists(directory):
            try:
                os.makedirs(directory)
            except OSError as e:  # folder may have been created by other threads
                if e.errno != 17:
                    print("Cannot create {0}".format(directory))
                    exit(1)
                pass


    def add_to_downloaded_log(self, img_filename):
        f = open(self.downloaded_log, "a")
        f.write("{0}\n".format(img_filename))
        f.close()


    def was_downloaded(self, img_filename):
        if os.path.isfile(self.downloaded_log):
            f = open(self.downloaded_log, "r")
            if str(img_filename) in f.read():
                f.close()
                return True
            else:
                return False
        else:
            return False


    def extension_condition(self, condition_ext, post_ext):
        if condition_ext:
            for extension in condition_ext:
                if extension == post_ext:
                    return True
        else:
            # Always return true if condition was not specified
            return True

        return False


    def filename_condition(self, condition_filename, post_filename):
        if condition_filename:
            for i in condition_filename:
                if i.lower() in post_filename.lower():
                    return True
        else:
            # Always return true if condition was not specified
            return True

        return False


    def width_condition(self, condition_width, post_width):
        if condition_width:
            if condition_width[0] == "=":
                if int(post_width) == int(condition_width.split("=")[-1]):
                    return True
            elif condition_width[0] == "<":
                if int(post_width) < int(condition_width.split("<")[-1]):
                    return True
            elif condition_width[0] == ">":
                if int(post_width) > int(condition_width.split(">")[-1]):
                    return True
            else:
                print("width need to be in this format:")
                print(">1024, <256 or =1920")
                exit(1)
        else:
            # Always return true if condition was not specified
            return True

        return False


    def height_condition(self, condition_height, post_height):
        if condition_height:
            if condition_height[0] == "=":
                if int(post_height) == int(condition_height.split("=")[-1]):
                    return True
            elif condition_height[0] == "<":
                if int(post_height) < int(condition_height.split("<")[-1]):
                    return True
            elif condition_height[0] == ">":
                if int(post_height) > int(condition_height.split(">")[-1]):
                    return True
            else:
                print("height need to be in this format:")
                print(">1024, <256 or =1080")
                exit(1)
        else:
            # Always return true if condition was not specified
            return True

        return False


    # Check if all condition returned true
    def all_condition_check(self, condition_list):
        for i in condition_list:
            if not i:
                return False
        return True


    # Return True if an image fit all search conditions
    def meet_dl_condition(self, post):
        condition_list = []
        condition_list.append(self.extension_condition(self.condition["ext"], post['ext']))
        condition_list.append(self.width_condition(self.condition["width"], post['w']))
        condition_list.append(self.height_condition(self.condition["height"], post['h']))
        condition_list.append(self.filename_condition(self.condition["filename"], post['filename']))
        return self.all_condition_check(condition_list)


    def remove_if_duplicate(self, img_path):
        if img_path:
            img_hash = dupecheck.hash_image(img_path)
            if dupecheck.is_duplicate(img_hash):
                os.remove(img_path)
                return True
            else:
                dupecheck.add_to_db(img_hash, self.thread_nb)
                return False


    def remove_tmp_files(self):
        if os.path.isfile(self.downloaded_log):
            os.unlink(self.downloaded_log)


    # Return downloaded picture path or false if an error occured
    def download_image(self, post_dic):
        try:
            pic_url = self.image_url + str(post_dic["tim"]) + post_dic["ext"]
            out_pic = os.path.join(self.tmp_dir, str(post_dic["tim"]) + post_dic["ext"])
            urllib.request.urlretrieve(pic_url, out_pic)
        except urllib.error.HTTPError as err:
            return False

        return out_pic



#!/usr/bin/env python3

import hashlib
import os
import sqlite3
from scanner.config import DB_FILE


def hash_image(img_location):
    with open(img_location, 'rb') as img:
        m = hashlib.md5()
        while True:
            data = img.read(8192)
            if not data:
                break
            m.update(data)
        return m.hexdigest()


def add_to_db(img_hash, thread_nb):
    conn = sqlite3.connect(DB_FILE)
    c = conn.cursor()

    c.execute("INSERT INTO Image_Hash (hash, Thread_Number) VALUES (?,?)", (img_hash, thread_nb))

    conn.commit()
    conn.close()


def is_duplicate(img_hash):
    conn = sqlite3.connect(DB_FILE)
    c = conn.cursor()

    c.execute("SELECT Hash FROM Image_Hash WHERE Hash = ?", (img_hash,))
    result = c.fetchone()

    conn.close()

    if result:
        return True
    else:
        return False



#!/usr/bin/env python3

import time
import json
import os
import re
from scanner import downloader, imageboard_info
from scanner.config import DB_FILE, currently_downloading
import sqlite3
import subprocess
import urllib.request
import threading
import http.client


def get_catalog_json(board, chan):
    chan_base_url = imageboard_info.imageboard_info(chan).base_url
    catalog = urllib.request.urlopen(
              "{0}{1}/catalog.json".format(chan_base_url, board))
    try:
        catalog_data = catalog.read()
    except http.client.IncompleteRead as err:
        catalog_data = err.partial
    return json.loads(catalog_data.decode("utf8"))


def scan_thread(keyword, catalog_json):
    # Check each thread, threads who contains the keyword are returned
    matched_threads = []
    for i in range(len(catalog_json)):
        for thread in catalog_json[i]["threads"]:
            regex = r'\b{0}\b'.format(keyword)
            # Search thread title
            if 'sub' in thread:
                if re.search(regex, str(thread["sub"]), re.IGNORECASE):
                    matched_threads.append(thread["no"])

            # Search OPs post body
            if 'com' in thread:
                if re.search(regex, str(thread["com"]), re.IGNORECASE):
                    matched_threads.append(thread["no"])

    return matched_threads


def download_thread(thread_id, chan, board, folder, output, condition, dupe_check):
    thread_downloader = downloader.downloader(thread_id, board,chan, output, folder, True, condition, dupe_check)
    t = threading.Thread(target=thread_downloader.download)
    t.daemon = True
    t.start()


def was_downloaded(thread_nb):
    conn = sqlite3.connect(DB_FILE)
    c = conn.cursor()

    c.execute("SELECT Thread_Number FROM Downloaded_Thread WHERE Thread_Number = ?", (thread_nb,))
    result = c.fetchone()

    conn.close()

    if result:
        return True
    else:
        return False

def folder_size_mb(folder):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(folder):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            total_size += os.path.getsize(fp)
    return total_size / 1000000


def check_quota(folder, quota_mb):
    if int(quota_mb) < folder_size_mb(os.path.join(folder, "downloads")):
        print("Quota limit exceeded. Stopping 4scanner.")
        exit(0)


def get_check_duplicate(search):
    if 'check_duplicate' in search:
        if search['check_duplicate']:
            return True
        else:
            return False

    # duplicate check is on by default
    return True

def get_condition(search):
    condition = {}
    if 'extension' in search:
        condition["ext"] = []
        if isinstance(search['extension'], str):
            condition["ext"].append(search['extension'])
        else:
            for extension in search['extension']:
                condition["ext"].append(extension)
    else:
        condition["ext"] = False

    if 'filename' in search:
        condition["filename"] = []
        if isinstance(search['filename'], str):
            condition["filename"].append(search['filename'])
        else:
            for extension in search['filename']:
                condition["filename"].append(extension)
    else:
        condition["filename"] = False

    if 'width' in search:
        condition["width"] = search['width']
    else:
        condition["width"] = False

    if 'height' in search:
        condition["height"] = search['height']
    else:
        condition["height"] = False

    return condition


def get_imageboard(search):
    if 'imageboard' in search:
        chan = search["imageboard"]
        # will raise error if nor supported
        imageboard_info.imageboard_info(chan)
    else:
        # default
        chan = "4chan"

    return chan


def get_keyword(search):
    if 'keywords' in search:
        keywords_array = []
        if isinstance(search['keywords'], str):
            keywords_array.append(search['keywords'])
        else:
            for keywords in search['keywords']:
                keywords_array.append(keywords)
    else:
        print("Cannot scan without any keyword...")
        exit(1)

    return keywords_array


def scan(keywords_file, output, quota_mb, wait_time):
    while True:
        if quota_mb:
            check_quota(output, quota_mb)

        curr_time = time.strftime('%d%b%Y-%H%M%S')
        print("{0} Searching threads...".format(curr_time))

        try:
            json_file = json.load(open(keywords_file))
        except ValueError:
            print("Your JSON file is malformed. Quitting.")
            exit(1)

        for search in json_file["searches"]:
            # Getting imageboard to search
            chan = get_imageboard(search)
            # Checking conditions
            condition = get_condition(search)
            # Check if we need to check for duplicate when downloading
            dupe_check = get_check_duplicate(search)
            # Getting output folder name
            folder_name = search["folder_name"]
            board = search["board"]
            keywords = get_keyword(search)

            try:
                catalog_json = get_catalog_json(board, chan)

                for keyword in keywords:
                    threads_id = scan_thread(keyword, catalog_json)

                    for thread_id in list(set(threads_id)):
                        if thread_id not in currently_downloading and not was_downloaded(thread_id):
                            download_thread(thread_id, chan, board,
                                            folder_name,
                                            output,
                                            condition, dupe_check)
                        # Used to keep track of what is currently downloading
                        currently_downloading.append(thread_id)
            except urllib.error.HTTPError as err:
                print("Error while opening {0} catalog page. "
                      "Retrying during next scan.".format(board))
                pass

        active_downloads = threading.active_count()-1
        print("{0} threads currently downloading.".format(active_downloads))
        print("Searching again in {0} minutes!"
              .format(str(int(wait_time / 60))))
        time.sleep(wait_time)



import os
from scanner import dupecheck
from scanner.config import DB_FILE
import sqlite3

# Initialize the DB used to store image hash and downloaded threads
def db_init():
    conn = sqlite3.connect(DB_FILE)
    c = conn.cursor()

    c.execute('''CREATE TABLE IF NOT EXISTS Image_Hash
             (Hash TEXT, Thread_Number INTEGER, Date_Added INTEGER DEFAULT (strftime('%s','now')));''')
    # TRY THIS TO FIX THE DATE (datetime('now','localtime')
    c.execute('''CREATE TABLE IF NOT EXISTS Downloaded_Thread
             (Thread_Number INTEGER, Imageboard TEXT, Board TEXT, Date_Added INTEGER DEFAULT (strftime('%s','now')));''')

    conn.commit()
    conn.close()


# Create home config directory
def create_conf_dir():
    if not os.path.isdir(os.path.expanduser("~/.4scanner")):
        os.mkdir(os.path.expanduser("~/.4scanner"))

create_conf_dir()
db_init()
